@@ -1,4 +1,4 @@
-/*  $Id: bounds.pl,v 1.12 2005/11/12 13:33:56 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -85,34 +85,41 @@
 		lex_chain/1,
 		indomain/1,
 		check/1,
-		tuples_in/2
+		tuples_in/2,
+		serialized/2
 	]).
 
 :- use_module(library('clp/clp_events')).
 
+/** <module> Simple integer solver that keeps track of upper and lower bounds
+
+@deprecated	No longer maintained.  Please use clpfd.pl
+@author		Tom Schrijvers
+*/
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % exported predicates
 X #>= Y :-
 	parse_expression(X,RX),
-	parse_expression(Y,RY), 
+	parse_expression(Y,RY),
 	geq(RX,RY,yes).
-X #=< Y :- 
+X #=< Y :-
 	parse_expression(X,RX),
 	parse_expression(Y,RY),
 	leq(RX,RY,yes).
 X #= Y :-
 	parse_expression(X,RX),
-	parse_expression(Y,RX). 
+	parse_expression(Y,RX).
 X #\= Y :-
 	parse_expression(X,RX),
-	parse_expression(Y,RY), 
+	parse_expression(Y,RY),
 	neq(RX,RY,yes).
 X #> Y :-
 	Z #= Y + 1,
 	X #>= Z.
 X #< Y :-
 	Y #> X.
-X in L .. U :- 
+X in L .. U :-
 	( is_list(X) ->
 		domains(X,L,U)
 	;
@@ -136,7 +143,7 @@ L #/\ R :-
 L #\/ R :-
 	reify(L,BL),
 	reify(R,BR),
-	myor(BL,BR,1).	
+	myor(BL,BR,1).
 L #\ R :-
 	reify(L,BL),
 	reify(R,BR),
@@ -250,10 +257,10 @@ parse_expression(Expr,Result) :-
 	; Expr = L mod R ->
 		parse_expression(L,RL),
 		parse_expression(R,RR),
-		mymod(RL,RR,Result)
+		mymod(RL,RR,Result,yes)
 	; Expr = abs(E) ->
 		parse_expression(E,RE),
-		myabs(RE,Result)
+		myabs(RE,Result,yes)
 	; Expr = (L / R) ->
 		parse_expression(L,RL),
 		parse_expression(R,RR),
@@ -269,16 +276,26 @@ indomain(Var) :-
 		true
 	).
 
-label(Vs) :- labeling([],Vs).
+label(Vs) :- labeling([], Vs).
 
-labeling(Options,Vars) :-
-	label(Options,leftmost,Vars).
+labeling(Options, Vars) :-
+	label(Options, leftmost, none, Vars).
+
+label([Option|Options], Selection, Optimisation, Vars) :-
+	( selection(Option) ->
+		label(Options, Option, Optimisation, Vars)
+	; optimization(Option) ->
+		label(Options, Selection, Option, Vars)
+	;
+		label(Options, Selection, Optimisation, Vars)
+	).
+label([], Selection, Optimisation, Vars) :-
+	( Optimisation == none ->
+		label(Vars, Selection)
+	;
+		optimize(Vars, Selection, Optimisation)
+	).
 
-label([Option|Options],_Selection,Vars) :-
-	selection(Option),
-	label(Options,Option,Vars).
-label([],Selection,Vars) :-
-	label(Vars,Selection).
 
 label([],_) :- !.
 label(Vars,Selection) :-
@@ -291,6 +308,8 @@ selection(min).
 selection(max).
 selection(leftmost).
 
+optimization(min(_)).
+optimization(max(_)).
 
 select_var(leftmost,[Var|Vars],Var,Vars).
 select_var(min,[V|Vs],Var,RVars) :-
@@ -314,33 +333,33 @@ find_min([V|Vs],CM,Min) :-
 find_max([],Var,Var).
 find_max([V|Vs],CM,Max) :-
 	( max_gt(V,CM) ->
-		find_min(Vs,V,Max)
+		find_max(Vs,V,Max)
 	;
-		find_min(Vs,CM,Max)
+		find_max(Vs,CM,Max)
 	).
 
 find_ff([],Var,Var).
 find_ff([V|Vs],CM,FF) :-
 	( ff_lt(V,CM) ->
-		find_min(Vs,V,FF)
+		find_ff(Vs,V,FF)
 	;
-		find_min(Vs,CM,FF)
+		find_ff(Vs,CM,FF)
 	).
 
 ff_lt(X,Y) :-
 	bounds(X,LX,UX),
 	bounds(Y,LY,UY),
-	UX - LX < UY - LY.	
+	UX - LX < UY - LY.
 
 min_lt(X,Y) :-
 	bounds(X,LX,_),
 	bounds(Y,LY,_),
-	LX < LY.	
+	LX < LY.
 
 max_gt(X,Y) :-
 	bounds(X,_,UX),
 	bounds(Y,_,UY),
-	UX > UY.	
+	UX > UY.
 
 bounds(X,L,U) :-
 	( var(X) ->
@@ -348,7 +367,7 @@ bounds(X,L,U) :-
 	;
 		X = L,
 		X = U
-	).	
+	).
 
 delete_eq([],_,[]).
 delete_eq([X|Xs],Y,List) :-
@@ -358,6 +377,29 @@ delete_eq([X|Xs],Y,List) :-
 		List = [X|Tail],
 		delete_eq(Xs,Y,Tail)
 	).
+
+optimize(Vars, Selection, Opt) :-
+	copy_term(Vars-Opt, Vars1-Opt1),
+	once(label(Vars1, Selection)),
+	functor(Opt1, Direction, _),
+	maplist(arg(1), [Opt,Opt1], [Expr,Expr1]),
+	optimize(Direction, Selection, Vars1, Vars, Expr1, Expr).
+
+optimize(Direction, Selection, Vars0, Vars, Expr0, Expr) :-
+	Val0 is Expr0,
+	copy_term(Vars-Expr, Vars1-Expr1),
+	( Direction == min ->
+		Tighten = (Expr1 #< Val0)
+	; % max
+		Tighten = (Expr1 #> Val0)
+	),
+	( Tighten, label(Vars1, Selection) ->
+		optimize(Direction, Selection, Vars1, Vars, Expr1, Expr)
+	;
+		Vars = Vars0,
+		Expr = Expr0
+	).
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 all_different([]).
 all_different([X|Xs]) :-
@@ -435,7 +477,7 @@ geq(X,Y,New) :-
 	; nonvar(X) ->
 		( nonvar(Y) ->
 			X >= Y
-		; 
+		;
 			get(Y,YL,YU,Exp) ->
 			NYU is min(X,YU),
 			put(Y,YL,NYU,Exp)
@@ -453,10 +495,10 @@ geq(X,Y,New) :-
 		; XU == YL ->
 			X = Y
 		; member(leq(Z,State),ExpX),
-	          Y == Z -> 
+	          Y == Z ->
 			set_passive(State),
-			X = Y	
-		; 
+			X = Y
+		;
 			( New == yes ->
 				active_state(State),
 				put(Y,YL,YU,[leq(X,State)|ExpY]),
@@ -474,7 +516,7 @@ geq(X,Y,New) :-
 			)
 		)
 	).
-		
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 neq(X,Y,New) :-
 	X \== Y,
@@ -499,7 +541,7 @@ neq(X,Y,New) :-
 					put(Y,L,U,[neq(X,State)|Exp])
 				;
 					true
-				)	
+				)
 			)
 		)
 	; nonvar(Y) ->
@@ -544,7 +586,7 @@ myplus(X,Y,Z,New) :-
 			put(Z,NZL,NZU,ZExp1),
 			( get(Y,YL2,YU2,YExp2) ->
 				NYL is max(YL,NZL-X),
-				NYU is min(YU,NZU-X),	
+				NYU is min(YU,NZU-X),
 				put(Y,NYL,NYU,YExp2)
 			;
 				Z is X + Y
@@ -628,7 +670,7 @@ div(X,Y,Z) :-
 		max_inf(Z)
 	; X < 0 ->
 		min_inf(Z)
-	).	
+	).
 
 mytimes(X,Y,Z,New) :-
 	( nonvar(X) ->
@@ -659,7 +701,7 @@ mytimes(X,Y,Z,New) :-
 				put(Y,NYL,NYU,YExp2)
 			;
 				Z is X * Y
-			)		
+			)
 		)
 	; nonvar(Y) ->
 		mytimes(Y,X,Z,New)
@@ -669,7 +711,7 @@ mytimes(X,Y,Z,New) :-
 		min_divide(Z,Z,YL,YU,TNXL),
 		max_divide(Z,Z,YL,YU,TNXU),
 		NXL is max(XL,ceiling(TNXL)),
-		NXU is min(XU,floor(TNXU)),		
+		NXU is min(XU,floor(TNXU)),
 		( New == yes ->
 			YExp1 = [mytimes(X,Z)|YExp],
 			put(Y,YL,YU,YExp1),
@@ -711,37 +753,37 @@ mytimes(X,Y,Z,New) :-
 			NYL is max(YL2,ceiling(TYL)),
 			max_divide(ZL,ZU,XL,XU,TYU),
 			NYU is min(YU2,floor(TYU)),
-			put(Y,NYL,NYU,YExp2)	
+			put(Y,NYL,NYU,YExp2)
 		;
 			NYL = Y,
 			NYU = Y
 		),
 		( get(Z,ZL2,ZU2,ZExp2) ->
-			min_times(NXL,NXU,NYL,NYU,TZL),	
+			min_times(NXL,NXU,NYL,NYU,TZL),
 			NZL is max(ZL2,TZL),
-			max_times(NXL,NXU,NYL,NYU,TZU),	
+			max_times(NXL,NXU,NYL,NYU,TZU),
 			NZU is min(ZU2,TZU),
 			put(Z,NZL,NZU,ZExp2)
 		;
 
 			true
-		)	
+		)
 	).
 
 max_times(L1,U1,L2,U2,Max) :-
-	Max is max(max(L1*L2,L1*U2),max(U1*L2,U1*U2)).	
+	Max is max(max(L1*L2,L1*U2),max(U1*L2,U1*U2)).
 min_times(L1,U1,L2,U2,Min) :-
-	Min is min(min(L1*L2,L1*U2),min(U1*L2,U1*U2)).	
+	Min is min(min(L1*L2,L1*U2),min(U1*L2,U1*U2)).
 max_divide(L1,U1,L2,U2,Max) :-
 	( L2 =< 0 , U2 >= 0 ->
 		max_inf(Max)
 	;
 		Max is max(max(div(L1,L2),div(L1,U2)),max(div(U1,L2),div(U1,U2)))
-	).	
+	).
 min_divide(L1,U1,L2,U2,Min) :-
 	( L2 =< 0 , U2 >= 0 ->
 		min_inf(Min)
-	;	
+	;
 		Min is min(min(div(L1,L2),div(L1,U2)),min(div(U1,L2),div(U1,U2)))
 	).
 
@@ -761,7 +803,7 @@ mydiv(X,Y,Z,New) :-
 			( nonvar(Z) ->
 				% TODO: cover this
 				true
-			;	
+			;
 				get(Z,ZL,ZU,ZExp),
 				( YL =< 0, YU >= 0 ->
 					NZL is max(-abs(X),ZL),
@@ -853,7 +895,7 @@ mymax(X,Y,Z,New) :-
 		( nonvar(Y) ->
 			Z is max(X,Y)
 		; nonvar(Z) ->
-			( Z == X ->		
+			( Z == X ->
 				geq(X,Y,yes)
 			; Z > X ->
 				Y = Z
@@ -870,9 +912,9 @@ mymax(X,Y,Z,New) :-
 			; X < ZL ->
 				Y = Z
 			;
-				( New == yes -> 
-					put(Y,YL,YU,[mymax(X,Z)|YExp]) 
-				; 
+				( New == yes ->
+					put(Y,YL,YU,[mymax(X,Z)|YExp])
+				;
 					true
 				),
 				NZL is max(ZL,X),
@@ -957,7 +999,7 @@ mymax(X,Y,Z,New) :-
 					mymax(Y,X,Z,no)
 				)
 			)
-		; 
+		;
 			mymax(X,Y,Z,no)
 		)
 	).
@@ -967,7 +1009,7 @@ mymin(X,Y,Z,New) :-
 		( nonvar(Y) ->
 			Z is min(X,Y)
 		; nonvar(Z) ->
-			( Z == X ->		
+			( Z == X ->
 				leq(X,Y,yes)
 			; Z < X ->
 				Y = Z
@@ -984,9 +1026,9 @@ mymin(X,Y,Z,New) :-
 			; X > ZU ->
 				Y = Z
 			;
-				( New == yes -> 
-					put(Y,YL,YU,[mymin(X,Z)|YExp]) 
-				; 
+				( New == yes ->
+					put(Y,YL,YU,[mymin(X,Z)|YExp])
+				;
 					true
 				),
 				NZL is max(ZL,YL),
@@ -1071,12 +1113,12 @@ mymin(X,Y,Z,New) :-
 					mymin(Y,X,Z,no)
 				)
 			)
-		; 
+		;
 			mymin(X,Y,Z,no)
 		)
 	).
 
-myabs(X,Y) :-
+myabs(X,Y,New) :-
 	( nonvar(X) ->
 		Y is abs(X)
 	; nonvar(Y) ->
@@ -1088,20 +1130,32 @@ myabs(X,Y) :-
 			X = Y
 		;
 			NLX is (-Y),
-			put(X,NLX,Y,[myabs(Y)|ExpX])
+			( New == yes ->
+				put(X,NLX,Y,[myabs(Y)|ExpX])
+			;
+				put(X,NLX,Y,ExpX)
+			)
 		)
 	;
 		get(X,LX,UX,ExpX),
 		get(Y,LY,UY,ExpY),
 		UY > 0,
-		put(X,LX,UX,[myabs(Y)|ExpX]),
+		( New == yes ->
+			put(X,LX,UX,[myabs(Y)|ExpX])
+		;
+			true
+		),
 		( LX =< 0, UX >= 0 ->
 			NLY is max(0,LY)
 		;
 			NLY is max(min(abs(LX),abs(UX)),max(LY,0))
 		),
 		NUY is min(UY,max(abs(LX),abs(UX))),
-		put(Y,NLY,NUY,[myabs2(X)|ExpY]),
+		( New == yes ->
+			put(Y,NLY,NUY,[myabs2(X)|ExpY])
+		;
+			put(Y,NLY,NUY,ExpY)
+		),
 		( var(X) ->
 			get(X,LX2,UX2,ExpX2),
 			( LX2 == LX, UX2 == UX ->
@@ -1116,7 +1170,7 @@ myabs(X,Y) :-
 					( UX < NLY ->
 						NUX is min((-NLY),UX)
 					;
-						NUX is min(NUY,UX)	
+						NUX is min(NUY,UX)
 					)
 				),
 				put(X,NLX,NUX,ExpX2)
@@ -1128,7 +1182,7 @@ myabs(X,Y) :-
 		)
 	).
 
-mymod(X,Y,Z) :- % Z is X mod Y
+mymod(X,Y,Z,New) :- % Z is X mod Y
 	( nonvar(X) ->
 		( nonvar(Y) ->
 			Z is X mod Y
@@ -1137,36 +1191,64 @@ mymod(X,Y,Z) :- % Z is X mod Y
 			( Z > 0 ->
 				( X == Z ->
 					no_overlap(-X,X,YL,YU,NYL,NYU),
-					put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					( New == yes ->
+						put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					;
+						put(Y,NYL,NYU,YExp)
+					)
 				; % Z < X
 					no_overlap(-Z,Z,YL,YU,YL1,YU1),
 					in_between(-X,X,YL1,YU1,NYL,NYU),
-					put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					( New == yes ->
+						put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					;
+						put(Y,NYL,NYU,YExp)
+					)
 				)
 			; Z < 0 ->
 				( X == Z ->
 					no_overlap(X,-X,YL,YU,NYL,NYU),
-					put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					( New == yes ->
+						put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					;
+						put(Y,NYL,NYU,YExp)
+					)
 				; % Z < X
 					no_overlap(Z,-Z,YL,YU,YL1,YU1),
 					in_between(X,-X,YL1,YU1,NYL,NYU),
-					put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					( New == yes ->
+						put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					;
+						put(Y,NYL,NYU,YExp)
+					)
 				)
 			; % Z == 0
 				( X == 0 ->
 					no_overlap(0,0,YL,YU,NYL,NYU),% Y \== 0
-					put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					( New == yes ->
+						put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					;
+						put(Y,NYL,NYU,YExp)
+					)
 				; % X \== 0
 					PX is abs(X),
 					in_between(-PX,PX,YL,YU,NYL,NYU),% |Y| < |X|
-					put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					( New == yes ->
+						put(Y,NYL,NYU,[mymod2(X,Z)|YExp])
+					;
+						put(Y,NYL,NYU,YExp)
+					)
 				)
 			)
 		; % var(Y), var(Z)
-			get(Y,YL,YU,YExp),
-			get(Z,ZL,ZU,ZExp),
-			put(Y,YL,YU,[mymod2(X,Z)|YExp]),
-			put(Z,ZL,ZU,[mymod3(X,Y)|ZExp])
+			( New == yes ->
+				get(Y,YL,YU,YExp),
+				get(Z,ZL,ZU,ZExp),
+				put(Y,YL,YU,[mymod2(X,Z)|YExp]),
+				put(Z,ZL,ZU,[mymod3(X,Y)|ZExp])
+			;
+				true
+			)
 		)
 	; nonvar(Y) ->
 		Y \== 0,
@@ -1174,33 +1256,57 @@ mymod(X,Y,Z) :- % Z is X mod Y
 			get(X,XL,XU,XExp),
 			( Z > 0 ->
 				NXL is max(XL,1),
-				put(X,NXL,XU,[mymod1(Y,Z)|XExp])
+				( New == yes ->
+					put(X,NXL,XU,[mymod1(Y,Z)|XExp])
+				;
+					put(X,NXL,XU,XExp)
+				)
 			; Z < 0 ->
 				NXU is min(XU,-1),
-				put(X,XL,NXU,[mymod1(Y,Z)|XExp])
+				( New == yes ->
+					put(X,XL,NXU,[mymod1(Y,Z)|XExp])
+				;
+					put(X,XL,NXU,XExp)
+				)
 			;
-				put(X,XL,XU,[mymod1(Y,Z)|XExp])	
+				( New == yes ->
+					put(X,XL,XU,[mymod1(Y,Z)|XExp])
+				;
+					put(X,XL,XU,XExp)
+				)
 			)
 		;
-			get(X,XL,XU,XExp),
-			get(Z,ZL,ZU,ZExp),
-			put(X,XL,XU,[mymod1(Y,Z)|XExp]),
-			put(Z,ZL,ZU,[mymod3(X,Y)|ZExp])
+			( New == yes ->
+				get(X,XL,XU,XExp),
+				get(Z,ZL,ZU,ZExp),
+				put(X,XL,XU,[mymod1(Y,Z)|XExp]),
+				put(Z,ZL,ZU,[mymod3(X,Y)|ZExp])
+			;
+				true
+			)
 		)
 	; nonvar(Z) ->
-		get(X,XL,XU,XExp),
-		get(Y,YL,YU,YExp),
-		put(X,XL,XU,[mymod1(Y,Z)|XExp]),
-		put(Y,YL,YU,[mymod2(X,Z)|YExp])
+		( New == yes ->
+			get(X,XL,XU,XExp),
+			get(Y,YL,YU,YExp),
+			put(X,XL,XU,[mymod1(Y,Z)|XExp]),
+			put(Y,YL,YU,[mymod2(X,Z)|YExp])
+		;
+			true
+		)
 	;
-		get(X,XL,XU,XExp),
-		get(Y,YL,YU,YExp),
-		get(Z,ZL,ZU,ZExp),
-		put(X,XL,XU,[mymod1(Y,Z)|XExp]),
-		put(Y,YL,YU,[mymod2(X,Z)|YExp]),
-		put(Z,ZL,ZU,[mymod3(X,Y)|ZExp])
+		( New == yes ->
+			get(X,XL,XU,XExp),
+			get(Y,YL,YU,YExp),
+			get(Z,ZL,ZU,ZExp),
+			put(X,XL,XU,[mymod1(Y,Z)|XExp]),
+			put(Y,YL,YU,[mymod2(X,Z)|YExp]),
+			put(Z,ZL,ZU,[mymod3(X,Y)|ZExp])
+		;
+			true
+		)
 	).
-			
+
 no_overlap(XL,XU,YL,YU,NYL,NYU) :-
 	( XL =< YL ->
 		NYL is XU + 1,
@@ -1280,7 +1386,7 @@ reified_geq(X,Y,B) :-
 			)
 		)
 	; B == 1 ->
-		X #>= Y	
+		X #>= Y
 	; B == 0 ->
 		X #< Y
 	).
@@ -1322,7 +1428,7 @@ check_lt(X,Y) :-
 			UX < LY
 		)
 	).
-			
+
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 reified_eq(X,Y,B) :-
 	( var(B) ->
@@ -1373,7 +1479,7 @@ reified_eq(X,Y,B) :-
 	; B == 0 ->
 		X #\= Y
 	).
-		
+
 
 check_eq(X,Y) :-
 	X == Y.
@@ -1529,7 +1635,7 @@ put(X,L,U,Exp) :-
 	L =< U,
 	( L == U ->
 		X = L,
-		trigger_exps(Exp,X) 
+		trigger_exps(Exp,X)
 	;
 		( get_attr(X,bounds,Attr) ->
 			put_attr(X,bounds,bounds(L,U,Exp)),
@@ -1541,7 +1647,7 @@ put(X,L,U,Exp) :-
 				trigger_exps(Exp,X),
 				notify(X,bounds)
 			)
-		; 
+		;
 			%format('\t~w in ~w .. ~w\n',[X,L,U]),
 			put_attr(X,bounds,bounds(L,U,Exp)),
 			notify(X,bounds)
@@ -1571,7 +1677,7 @@ attr_unify_hook(bounds(L,U,Exp),Other) :-
 		NU is min(U,OU),
 		append(Exp,OExp,NExp),
 		check_neqs(NExp,Other),
-		put(Other,NL,NU,NExp)	
+		put(Other,NL,NU,NExp)
 	; % nonvar(Other) ->
 		Other >= L,
 		Other =< U,
@@ -1586,7 +1692,7 @@ check_neqs([E|Es],X) :-
 	;
 		check_neqs(Es,X)
 	).
-		
+
 
 trigger_exps([],_).
 trigger_exps([E|Es],X) :-
@@ -1667,16 +1773,16 @@ trigger_exp(myxor2(X,Y),Z) :-
 	myxor(X,Y,Z).
 
 trigger_exp(myabs(Y),X) :-
-	myabs(X,Y).
+	myabs(X,Y,no).
 trigger_exp(myabs2(X),Y) :-
-	myabs(X,Y).
+	myabs(X,Y,no).
 
 trigger_exp(mymod1(Y,Z),X) :-
-	mymod(X,Y,Z).
+	mymod(X,Y,Z,no).
 trigger_exp(mymod2(X,Z),Y) :-
-	mymod(X,Y,Z).
+	mymod(X,Y,Z,no).
 trigger_exp(mymod3(X,Y),Z) :-
-	mymod(X,Y,Z).
+	mymod(X,Y,Z,no).
 
 trigger_exp(mydiv(Y,Z),X) :-
 	mydiv(X,Y,Z,no).
@@ -1701,7 +1807,10 @@ trigger_exp(rel_tuple(Relation,Tuple), _) :-
 			)
 		)
 	).
-	
+
+trigger_exp(serialized(Duration,Left,Right), X) :-
+	myserialized(Duration, Left, Right, X).
+
 
 memberchk_eq(X,[Y|Ys],Z) :-
    (   X == Y ->
@@ -1803,6 +1912,109 @@ relation_unifiable([R|Rs], Tuple, Us) :-
 	),
 	relation_unifiable(Rs, Tuple, Rest).
 
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+    % serialized/2; see Dorndorf et al. 2000, "Constraint Propagation
+    % Techniques for the Disjunctive Scheduling Problem"
+
+    % attribute: serialized(Duration, Left, Right)
+    %   Left and Right are lists of Start-Duration pairs representing
+    %   other tasks occupying the same resource
+
+    % Currently implements 2-b-consistency
+
+serialized(Starts, Durations) :-
+	pair_up(Starts, Durations, SDs),
+	serialize(SDs, []).
+
+
+pair_up([], [], []).
+pair_up([A|As], [B|Bs], [A-B|ABs]) :- pair_up(As, Bs, ABs).
+
+
+    % store attributes for serialized/2 constraint
+
+serialize([], _).
+serialize([Start-Duration|SDs], Left) :-
+	( var(Start) ->
+		get(Start, L, U, Exp),
+		put(Start, L, U, [serialized(Duration,Left,SDs)|Exp])
+	;
+		true
+	),
+	myserialized(Duration, Left, SDs, Start),
+	serialize(SDs, [Start-Duration|Left]).
+
+    % consistency check / propagation
+
+myserialized(Duration, Left, Right, Start) :-
+	myserialized(Left, Start, Duration),
+	myserialized(Right, Start, Duration).
+
+
+earliest_start_time(Start, EST) :-
+	( var(Start) ->
+		get(Start, EST, _, _)
+	;
+		EST = Start
+	).
+
+latest_start_time(Start, LST) :-
+	( var(Start) ->
+		get(Start, _, LST, _)
+	;
+		LST = Start
+	).
+
+
+myserialized([], _, _).
+myserialized([S_I-D_I|SDs], S_J, D_J) :-
+	( var(S_I) ->
+		serialize_lower_bound(S_I, D_I, Start, D_J),
+		( var(S_I) -> % STILL variable?
+			serialize_upper_bound(S_I, D_I, Start, D_J)
+		;
+			true
+		)
+	; var(S_J) ->
+		serialize_lower_bound(S_J, D_J, S, D_I),
+		( var(S_J) ->
+			serialize_upper_bound(S_J, D_J, S, D_I)
+		;
+			true
+		)
+	;
+		( S_I + D_I =< S_J ->
+			true
+		; S_J + D_J =< S_I ->
+			true
+		;
+			fail
+		)
+	),
+	myserialized(SDs, S_J, D_J).
+
+serialize_lower_bound(I, D_I, J, D_J) :-
+	get(I, EST_I, U, Exp),
+	latest_start_time(J, LST_J),
+	( EST_I + D_I > LST_J ->
+		earliest_start_time(J, EST_J),
+		EST is max(EST_I, EST_J+D_J),
+		put(I, EST, U, Exp)
+	;
+		true
+	).
+
+serialize_upper_bound(I, D_I, J, D_J) :-
+	get(I, L, LST_I, Exp),
+	earliest_start_time(J, EST_J),
+	( EST_J + D_J > LST_I ->
+		latest_start_time(J, LST_J),
+		LST is min(LST_I, LST_J-D_I),
+		put(I, L, LST, Exp)
+	;
+		true
+	).
 
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
