@@ -34,6 +34,7 @@
 	  append/2,			% +ListOfLists, -List
 	  append/3,
 	  select/3,
+	  selectchk/3,
 	  nextto/3,			% ?X, ?Y, ?List
 	  delete/3,
 	  nth0/3,
@@ -43,6 +44,8 @@
 	  permutation/2,		% ?List, ?Permutation
 	  flatten/2,			% +Nested, -Flat
 	  sumlist/2,			% +List, -Sum
+	  max_list/2,			% +List, -Max
+	  min_list/2,			% +List, -Min
 	  numlist/3,			% +Low, +High, -List
 
 	  is_set/1,			% set manipulation
@@ -62,7 +65,7 @@ the DEC-10 Prolog library (LISTRO.PL). Contributed by Richard O'Keefe.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 %%	member(?Elem, ?List)
-%	
+%
 %	True if Elem is a member of List
 
 member(X, [X|_]).
@@ -70,7 +73,7 @@ member(X, [_|T]) :-
 	member(X, T).
 
 %%	append(?List1, ?List2, ?List1AndList2)
-%	
+%
 %	List1AndList2 is the concatination of List1 and List2
 
 append([], L, L).
@@ -81,7 +84,7 @@ append([H|T], L, [H|R]) :-
 %
 %	Concatenate a list of lists.  Is  true   if  Lists  is a list of
 %	lists, and List is the concatenation of these lists.
-%	
+%
 %	@param	ListOfLists must be a list of -possibly- partial lists
 
 append(ListOfLists, List) :-
@@ -103,8 +106,18 @@ select(Elem, [Head|Tail], [Head|Rest]) :-
 	select(Elem, Tail, Rest).
 
 
+%%	selectchk(+Elem, +List, -Rest) is semidet.
+%
+%	Semi-deterministic removal of first element in List that unifies
+%	Elem.
+
+selectchk(Elem, List, Rest) :-
+	select(Elem, List, Rest0), !,
+	Rest = Rest0.
+
+
 %%	nextto(?X, ?Y, ?List)
-%	
+%
 %	True of Y follows X in List.
 
 nextto(X, Y, [X,Y|_]).
@@ -117,7 +130,7 @@ nextto(X, Y, [_|Zs]) :-
 %	List2.
 
 delete([], _, []) :- !.
-delete([Elem|Tail], Elem, Result) :- !, 
+delete([Elem|Tail], Elem, Result) :- !,
 	delete(Tail, Elem, Result).
 delete([Head|Tail], Elem, [Head|Rest]) :-
 	delete(Tail, Elem, Rest).
@@ -175,7 +188,7 @@ nth1(Index, List, Elem) :-
 %%	last(?List, ?Last)
 %
 %	Succeeds if `Last' unifies with the last element of `List'.
-%	
+%
 %	@compat	There is no de-facto standard for the argument order of
 %		last/2.  Be careful when porting code or use
 %		append(_, [Last], List) as a portable alternative.
@@ -202,7 +215,7 @@ reverse([X|Xs], Rs, Ys, [_|Bound]) :-
 
 
 %%	permutation(?Xs, ?Ys) is nondet.
-%	
+%
 %	permutation(Xs, Ys) is true when Xs is a permutation of Ys. This
 %	can solve for Ys given Xs or Xs given Ys, or even enumerate Xs
 %	and Ys together.
@@ -218,14 +231,14 @@ permutation([X|Xs], Ys1, [_|Bound]) :-
 %%	flatten(+List1, ?List2) is det.
 %
 %	Is true it List2 is a non nested version of List1.
-%	
+%
 %	@deprecated	Ending up needing flatten/3 often indicates,
 %			like append/3 for appending two lists, a bad
 %			design.  Efficient code that generates lists
 %			from generated small lists must use difference
 %			lists, often possible through grammar rules for
 %			optimal readability.
-%	@see append/2		
+%	@see append/2
 
 flatten(List, FlatList) :-
 	flatten(List, [], FlatList0), !,
@@ -235,12 +248,12 @@ flatten(Var, Tl, [Var|Tl]) :-
 	var(Var), !.
 flatten([], Tl, Tl) :- !.
 flatten([Hd|Tl], Tail, List) :- !,
-	flatten(Hd, FlatHeadTail, List), 
+	flatten(Hd, FlatHeadTail, List),
 	flatten(Tl, Tail, FlatHeadTail).
 flatten(NonList, Tl, [NonList|Tl]).
 
 %%	sumlist(+List, -Sum) is det.
-%	
+%
 %	Sum is the result of adding all numbers in List.
 
 sumlist(Xs, Sum) :-
@@ -252,20 +265,50 @@ sumlist([X|Xs], Sum0, Sum) :-
 	sumlist(Xs, Sum1, Sum).
 
 
-%%	numlist(+Low, +High, -List) is det.
-%	
-%	List is a list [Low, Low+1, ... High]
+%%	max_list(+List:list(number), -Max:number) is det.
+%
+%	True if Max is the largest number in List.
+
+max_list([H|T], Max) :-
+	max_list(T, H, Max).
+
+max_list([], Max, Max).
+max_list([H|T], Max0, Max) :-
+	Max1 is max(H, Max0),
+	max_list(T, Max1, Max).
+
+
+%%	min_list(+List:list(number), -Min:number) is det.
+%
+%	True if Min is the largest number in List.
+
+min_list([H|T], Min) :-
+	min_list(T, H, Min).
+
+min_list([], Min, Min).
+min_list([H|T], Min0, Min) :-
+	Min1 is min(H, Min0),
+	min_list(T, Min1, Min).
+
+
+%%	numlist(+Low, +High, -List) is semidet.
+%
+%	List is a list [Low, Low+1, ... High].  Fails if High < Low.
+%
+%	@error type_error(integer, Low)
+%	@error type_error(integer, High)
 
 numlist(L, U, Ns) :-
-    integer(L), integer(U), L =< U,
-    numlist_(L, U, Ns).
+	must_be(integer, L),
+	must_be(integer, U),
+	L =< U,
+	numlist_(L, U, Ns).
 
+numlist_(U, U, List) :- !,
+	List = [U].
 numlist_(L, U, [L|Ns]) :-
-    (   L =:= U
-    ->  Ns = []
-    ;   M is L + 1,
-	numlist_(M, U, Ns)
-    ).
+	L2 is L+1,
+	numlist_(L2, U, Ns).
 
 
 		/********************************
@@ -273,13 +316,13 @@ numlist_(L, U, [L|Ns]) :-
 		*********************************/
 
 %%	is_set(@Set) is det.
-%	
+%
 %	True if Set is a proper list without duplicates.
 
 is_set(0) :- !, fail.		% catch variables
 is_set([]) :- !.
 is_set([H|T]) :-
-	memberchk(H, T), !, 
+	memberchk(H, T), !,
 	fail.
 is_set([_|T]) :-
 	is_set(T).
@@ -288,7 +331,10 @@ is_set([_|T]) :-
 %%	list_to_set(+List, ?Set) is det.
 %
 %	True when Set has the same element   as  List in the same order.
-%	The left-most copy of the duplicate is retained.
+%	The left-most copy of the duplicate  is retained. The complexity
+%	of this operation is |List|^2.
+%
+%	@see sort/2.
 
 list_to_set(List, Set) :-
 	list_to_set_(List, Set0),
@@ -297,7 +343,7 @@ list_to_set(List, Set) :-
 list_to_set_([], R) :-
 	close_list(R).
 list_to_set_([H|T], R) :-
-	memberchk(H, R), !, 
+	memberchk(H, R), !,
 	list_to_set_(T, R).
 
 close_list([]) :- !.
@@ -307,43 +353,55 @@ close_list([_|T]) :-
 
 %%	intersection(+Set1, +Set2, -Set3) is det.
 %
-%	True if Set3 unifies with the intersection of Set1 and Set2
+%	True if Set3 unifies with the intersection of Set1 and Set2.
+%	The complexity of this predicate is |Set1|*|Set2|
+%
+%	@see ord_intersection/3.
 
 intersection([], _, []) :- !.
 intersection([X|T], L, Intersect) :-
-	memberchk(X, L), !, 
-	Intersect = [X|R], 
+	memberchk(X, L), !,
+	Intersect = [X|R],
 	intersection(T, L, R).
 intersection([_|T], L, R) :-
 	intersection(T, L, R).
 
 
 %%	union(+Set1, +Set2, -Set3) is det.
-%	
-%	True if Set3 unifies with the union of Set1 and Set2
+%
+%	True if Set3 unifies with the union of Set1 and Set2.
+%	The complexity of this predicate is |Set1|*|Set2|
+%
+%	@see ord_union/3.
 
 union([], L, L) :- !.
 union([H|T], L, R) :-
-	memberchk(H, L), !, 
+	memberchk(H, L), !,
 	union(T, L, R).
 union([H|T], L, [H|R]) :-
 	union(T, L, R).
 
 
 %%	subset(+SubSet, +Set) is det.
-%	
-%	True if all elements of SubSet belong to Set as well.
+%
+%	True if all elements of SubSet belong to Set as well. Membership
+%	test is based on memberchk/2.  The complexity is |SubSet|*|Set|.
+%
+%	@see ord_subset/2.
 
 subset([], _) :- !.
 subset([E|R], Set) :-
-	memberchk(E, Set), 
+	memberchk(E, Set),
 	subset(R, Set).
 
 
 %%	subtract(+Set, +Delete, -Result) is det.
 %
-%	Delete all elements from `Set' that occur in `Delete' (a set) and
-%	unify the result with `Result'.
+%	Delete all elements from `Set' that   occur  in `Delete' (a set)
+%	and unify the  result  with  `Result'.   Deletion  is  based  on
+%	unification using memberchk/2. The complexity is |Delete|*|Set|.
+%
+%	@see ord_subtract/3.
 
 subtract([], _, []) :- !.
 subtract([E|T], D, R) :-
