@@ -6,8 +6,7 @@
     Copyright (C) 1999 SWI, University of Amsterdam. All rights reserved.
 */
 
-:- initialization
-	load_foreign_library(test).
+:- use_foreign_library(test).
 a(1).
 a(2).
 %a(a).
