@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -48,7 +48,7 @@ parms(test, [ user(jan) | Options ]) :-
 	->  Options = []
 	;   Options = [password(Pass)]
 	).
-	
+
 
 set_db(DSN, Options) :-
 	retractall(parms(_,_)),
@@ -78,7 +78,7 @@ types we test this too.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 %	type(SqlType, PlType=Values, [AltType=Map, ...], Options)
-%	
+%
 %	Define a test-set.  The first argument is the SQL type to test.
 %	PlType is the default Prolog type with a set of values.  AltType
 %	is an alternative type that can be used to exchange values and
@@ -91,9 +91,15 @@ type(integer,
        float = integer_to_float		% exchange as float
      ],
      []).
+type(bigint,
+     integer = [-1, 0, 42, 35184372088832, -35184372088832, '$null$' ],
+     [ atom  = integer_to_atom,		% exchange as text
+       float = integer_to_float		% exchange as float
+     ],
+     []).
 type(float,
      float   = [-1.0, 0.0, 42.0, 3.2747, '$null$' ],
-     [ 
+     [
      ],
      []).
 type(decimal(10,2),
@@ -147,7 +153,7 @@ type(varchar(2000),				% can we access as integers?
 	      Long,
 	      '$null$'
 	    ],
-     [ 
+     [
      ],
      [ \+ dbms_name('MySQL')
      ]) :-
@@ -243,7 +249,7 @@ db_type(Type,         Type, Comment) :-
 	sformat(Comment, 'Type ~w', [Type]).
 
 %	applicable(+Options, +Type)
-%	
+%
 %	See whether we can run this test on this connection.
 
 applicable([], _) :- !.
@@ -258,7 +264,7 @@ applicable(_, _).
 
 
 %	read_test_alt_types([Type=Map, ...], Table, Values)
-%	
+%
 %	Try to read the table using alternative Prolog types and check
 %	the results.
 
@@ -272,7 +278,7 @@ read_test_alt_type(Type, Map, Table, Values) :-
 	maplist(Map, Values, AltValues),
 	read_values(Table, Type, ReadValues),
 	compare_sets(AltValues, ReadValues).
-	
+
 %	write_test_alt_types([Type=Map, ...], Table, Values)
 %
 
@@ -291,7 +297,7 @@ write_test_alt_type(Type, Map, SqlType, Table) :-
 	compare_sets(NativeValues, ReadValues).
 
 %	insert_values(+Table, +OdbcType, +PlType, +Values)
-%	
+%
 %	Insert Prolog values into the table
 
 insert_values(Table, SqlType, PlType, Values) :-
@@ -326,11 +332,22 @@ compare_elements([H0|T0], [H1|T1]) :-
 	    float(H1)
 	->  Diff is H0-H1,
 	    format('~NERROR: ~q != ~q (diff=~f)~n', [H0, H1, Diff])
-	;   format('~NERROR: ~q != ~q~n', [H0, H1])
+	;   print_val(H0, P0),
+	    print_val(H1, P1),
+	    format('~NERROR: ~q != ~q~n', [P0, P1])
 	),
 	compare_elements(T0, T1).
 
 
+print_val(H, P) :-
+	atom(H),
+	atom_length(H, Len),
+	Len > 60, !,
+	sub_atom(H, 0, 40, _, Start),
+	sub_atom(H, _, 20, 0, End),
+	atomic_list_concat([Start, '...', End], P).
+print_val(H, H).
+
 		 /*******************************
 		 *	       MAPS		*
 		 *******************************/
@@ -511,7 +528,7 @@ fetch(Statement, Options) :-
 	;   writeln(Row),
 	    fetch(Statement, Options)
 	).
-	
+
 
 		 /*******************************
 		 *	       META		*
