@@ -38,6 +38,7 @@
 #endif
 
 #include "ssllib.h"
+#include <openssl/rand.h>
 
 #ifdef __SWI_PROLOG__
 #include <SWI-Stream.h>
@@ -139,7 +140,7 @@ static SSL_SOCK_STATUS
 ssl_inspect_status(SSL *ssl, int sock, int ssl_ret)
 {   int code;
 
-    if (ssl_ret > 0) {
+    if (ssl_ret >= 0) {
         return SSL_SOCK_OK;
     }
 
@@ -373,7 +374,7 @@ ssl_config_free( void *            ctx
                , long  argl
                , void *argp
                )
-{   
+{
     PL_SSL *config = NULL;
 
     ssl_deb(1, "calling ssl_config_free()\n");
@@ -526,7 +527,7 @@ ssl_cb_cert_verify(int preverify_ok, X509_STORE_CTX *ctx)
 {
     SSL    * ssl    = NULL;
     PL_SSL * config = NULL;
-    
+
     /*
      * Get our config data
      */
@@ -821,7 +822,7 @@ ssl_config(PL_SSL *config)
                                  , ssl_cb_pem_passwd
                                  ) ;
     ssl_deb(1, "password handler installed\n");
-   
+
     if (config->pl_ssl_cert_required) {
         if (config->pl_ssl_certf == NULL ||
             config->pl_ssl_keyf  == NULL) {
@@ -847,7 +848,7 @@ ssl_config(PL_SSL *config)
         }
         ssl_deb(1, "certificate installed successfully\n");
     }
-    
+
     (void) SSL_CTX_set_verify( config->pl_ssl_ctx
                              , (config->pl_ssl_peer_cert_required)
                                ? SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT
@@ -879,6 +880,9 @@ ssl_lib_init(void)
  */
 {
     SSL_load_error_strings();
+    /* This call will ensure we only end up calling RAND_poll() once
+       - preventing an ugly synchronization issue in OpenSSL */
+    RAND_status();
     (void) SSL_library_init();
 
     if ((ctx_idx = SSL_CTX_get_ex_new_index( 0
@@ -928,7 +932,7 @@ nbio_fd() returns the corresponding file/socket descriptor.
 
 Probably this is fine, surely  for   Unix.  Nevertheless,  a much better
 solution would be to wrap the nbio_* functions into an SSL BIO. See "man
-BIO_new", etc. 
+BIO_new", etc.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 PL_SSL_INSTANCE *
@@ -1005,10 +1009,10 @@ ssl_ssl(PL_SSL *config, int sock_inst)
 	            /* success */
 	            ssl_deb(1, "established ssl client side\n");
 	            return instance;
-	            
+
 	         case SSL_SOCK_RETRY:
 	            continue;
-	            
+
 	         case SSL_SOCK_ERROR:
 	            if (SSL_get_error(instance->ssl, ssl_ret) == SSL_ERROR_WANT_READ)
 	            {
@@ -1331,7 +1335,7 @@ ssl_thread_setup(void)
   { lock_count[i]=0;
     pthread_mutex_init(&(lock_cs[i]), NULL);
   }
-			       
+
 #ifndef __WINDOWS__
   CRYPTO_set_id_callback(pthreads_thread_id);
 #endif
