@@ -1,4 +1,4 @@
-/*  $Id: pce_goal_expansion.pl,v 1.7 2006/03/30 20:59:32 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -32,7 +32,7 @@
 :- module(pce_goal_expansion, []).
 :- use_module(pce_realise).
 :- use_module(pce_boot(pce_expansion), [pce_compiling/1]).
-:- use_module(pce_boot(pce_principal), [get/3]).
+:- use_module(pce_boot(pce_principal)).
 :- require([ pce_error/1
 	   , append/3
 	   ]).
@@ -101,7 +101,7 @@ selector(Sel) :-
 	pce_error(error(type_error(selector, Sel), _)),
 	fail.
 
-%	current_super_class(-Super)
+%%	current_super_class(-Super)
 %
 %	Return the name of the super-class of the currently compiling
 %	class.
@@ -113,7 +113,7 @@ current_super_class(Op, _) :-
 	pce_error(context_error(Op, nosuper, goal)),
 	fail.
 
-%	super_class(+ClassName, -SuperClassName)
+%%	super_class(+ClassName, -SuperClassName)
 %
 %	Deternine the super-class of a class.  Try to avoid realising
 %	the class to improve compilation and save-state performance.
@@ -139,7 +139,7 @@ pce_ifhostproperty(prolog(sicstus),
 	expandable(Name, Arity),
 	expand(Goal, ExpandedGoal)
 ),
-(   user:goal_expansion(Goal, ExpandedGoal) :-
+(   system:goal_expansion(Goal, ExpandedGoal) :-
 	functor(Goal, Name, Arity),
 	expandable(Name, Arity),
 	expand(Goal, ExpandedGoal)
