@@ -40,6 +40,7 @@
 
 	    debug/2			% +Fmt, +Args
 	  ]).
+:- use_module(library(pce)).
 :- use_module(clause).
 
 		 /*******************************
@@ -54,6 +55,7 @@ setting(verbose, 	   false).	% debugging this package
 setting(active,	 	   true).	% actually use this tracer
 setting(show_unbound,	   false).	% show unbound variables
 setting(cluster_variables, true).	% cluster variables
+setting(attributes,        portray).	% How to print attributes
 setting(list_max_clauses,  25).		% only list this amount of clauses
 setting(stack_depth, 	   10).		% # frames shown
 setting(choice_depth,	   10).		% # choice-points shown
@@ -78,7 +80,7 @@ trace_setting(Name, Old, New) :-
 %
 %	Finds the source-location of the predicate.  If the predicate
 %	is not defined, it will list the predicate on @dynamic_source_buffer
-%	and return this buffer.	
+%	and return this buffer.
 
 find_source(Predicate, File, Line) :-
 	predicate_property(Predicate, file(File)),
@@ -114,7 +116,7 @@ list_predicate(Predicate) :-
 	    send(@dynamic_source_buffer, attribute, comment,
 		 string('Decompiled listing of %s', PrintName))
 	).
-	    
+
 
 list_clauses(Predicate, From, To) :-
 	between(From, To, Nth),
