@@ -289,6 +289,9 @@ ignore_file('plwin.opt').
 ignore_file('pl2xpce.pdb').
 ignore_file('double_metaphone.pdb').
 ignore_file('porter_stem.pdb').
+ignore_file('Support SWI-Prolog development.url').
+ignore_file('SWI-Prolog website.url').
+ignore_file('uninstall.exe').
 
 
 		 /*******************************
